@@ -1,5 +1,5 @@
 // Copyright Contributors to the OpenVDB Project
-// SPDX-License-Identifier: MPL-2.0
+// SPDX-License-Identifier: Apache-2.0
 
 /// @file compiler/PointExecutable.cc
 
@@ -17,6 +17,7 @@
 #include "openvdb_ax/codegen/Codecs.h"
 
 #include <openvdb/Types.h>
+#include <openvdb/util/Assert.h>
 
 #include <openvdb/points/AttributeArray.h>
 #include <openvdb/points/PointAttribute.h>
@@ -27,6 +28,8 @@
 #include <openvdb/points/PointMove.h>
 #include <openvdb/points/PointDelete.h>
 
+#include <llvm/Config/llvm-config.h>
+
 namespace openvdb {
 OPENVDB_USE_VERSION_NAMESPACE
 namespace OPENVDB_VERSION_NAME {
@@ -55,7 +58,7 @@ struct PointExecutable::Settings
 
     inline std::vector<cli::ParamBase*> optional()
     {
-        assert(IsCLI);
+        OPENVDB_ASSERT(IsCLI);
         std::vector<cli::ParamBase*> params {
             &this->mCreateMissing,
             &this->mGroup,
@@ -157,8 +160,8 @@ using KernelValueFunctionPtr = std::add_pointer<codegen::PointKernelAttributeArr
 using KernelBufferRangeFunctionPtr = std::add_pointer<codegen::PointKernelBufferRange::Signature>::type;
 using PointLeafLocalData = codegen::codegen_internal::PointLeafLocalData;
 
-#ifndef NDEBUG
-inline bool supported(const ast::tokens::CoreType type)
+#ifdef OPENVDB_ENABLE_ASSERTS
+[[maybe_unused]] inline bool supported(const ast::tokens::CoreType type)
 {
     switch (type) {
         case ast::tokens::BOOL    : return true;
@@ -331,7 +334,7 @@ struct PointFunctionArguments
         using FunctionTraitsT = codegen::PointKernelBufferRange::FunctionTraitsT;
         using ReturnT = FunctionTraitsT::ReturnType;
 
-        assert(mData.mUseBufferKernel);
+        OPENVDB_ASSERT(mData.mUseBufferKernel);
 
         return [&](const openvdb::Coord& origin, void* buffer, Index64* mask, const size_t size) -> ReturnT {
             mData.mKernelBufferRange(static_cast<FunctionTraitsT::Arg<0>::Type>(mData.mCustomData),
@@ -391,7 +394,7 @@ struct PointFunctionArguments
             const codegen::Codec* codec =
                 codegen::getCodec(ast::tokens::tokenFromTypeString(array.valueType()), array.codecType());
             if (codec) flag |= codec->flag();
-            assert(array.isDataLoaded() && !array.isUniform());
+            OPENVDB_ASSERT(array.isDataLoaded() && !array.isUniform());
         }
         else {
             typename WriteHandle<ValueT>::UniquePtr handle(new WriteHandle<ValueT>(leaf, Index(pos)));
@@ -404,14 +407,14 @@ struct PointFunctionArguments
 
     inline void addGroupHandle(const LeafT& leaf, const std::string& name)
     {
-        assert(leaf.attributeSet().descriptor().hasGroup(name));
+        OPENVDB_ASSERT(leaf.attributeSet().descriptor().hasGroup(name));
         mGroupHandles.emplace_back(new points::GroupHandle(leaf.groupHandle(name)));
         mVoidGroupHandles.emplace_back(static_cast<void*>(mGroupHandles.back().get()));
     }
 
     inline void addGroupWriteHandle(LeafT& leaf, const std::string& name)
     {
-        assert(leaf.attributeSet().descriptor().hasGroup(name));
+        OPENVDB_ASSERT(leaf.attributeSet().descriptor().hasGroup(name));
         mGroupHandles.emplace_back(new points::GroupWriteHandle(leaf.groupWriteHandle(name)));
         mVoidGroupHandles.emplace_back(static_cast<void*>(mGroupHandles.back().get()));
     }
@@ -422,7 +425,7 @@ struct PointFunctionArguments
     addAttributeHandle(LeafT& leaf, const std::string& name, const ast::tokens::CoreType type, const bool write)
     {
         // assert so the executer can be marked as noexcept (assuming nothing throws in compute)
-        assert(supported(type) && "Could not retrieve attribute handle from unsupported type");
+        OPENVDB_ASSERT(supported(type) && "Could not retrieve attribute handle from unsupported type");
         switch (type) {
             case ast::tokens::BOOL    : return this->addAttributeHandleTyped<bool>(leaf, name, write);
             case ast::tokens::CHAR    : return this->addAttributeHandleTyped<char>(leaf, name, write);
@@ -457,7 +460,7 @@ private:
     {
         const size_t pos = leaf.attributeSet().find(name);
         //assert(!leaf.attributeSet().isShared(pos));
-        assert(pos != openvdb::points::AttributeSet::INVALID_POS);
+        OPENVDB_ASSERT(pos != openvdb::points::AttributeSet::INVALID_POS);
         if (write) this->addWriteHandle<ValueType>(leaf, pos);
         else       this->addHandle<ValueType>(leaf, pos);
     }
@@ -503,7 +506,7 @@ struct PointExecuterDeformer
     void apply(Vec3d& position, const IterT& iter) const
     {
         if (mFilter.valid(iter)) {
-            assert(mPws);
+            OPENVDB_ASSERT(mPws);
             position = Vec3d(mPws->get(*iter));
         }
     }
@@ -647,7 +650,7 @@ void processAttributes(points::PointDataGrid& grid,
                        Logger& logger)
 {
     const auto leafIter = grid.tree().cbeginLeaf();
-    assert(leafIter);
+    OPENVDB_ASSERT(leafIter);
 
     attributeInfo.reserve(registry.data().size());
 
@@ -682,7 +685,7 @@ void processAttributes(points::PointDataGrid& grid,
 
         if (pos != points::AttributeSet::INVALID_POS) {
             const points::AttributeArray* const array = leafIter->attributeSet().getConst(pos);
-            assert(array);
+            OPENVDB_ASSERT(array);
             if (array->stride() > 1) {
                 logger.warning("Attribute \"" + name + (name != iter.name() ? "\" [bound to \"" + iter.name() + "\"]" : "\"")
                     + " on grid \"" + grid.getName() + "\"is a strided (array) attribute. "
@@ -704,7 +707,7 @@ void processAttributes(points::PointDataGrid& grid,
             continue;
         }
 
-        assert(supported(iter.type()));
+        OPENVDB_ASSERT(supported(iter.type()));
         const NamePair type = typePairFromToken(iter.type());
         points::appendAttribute(grid.tree(), name, type);
     }
@@ -742,22 +745,33 @@ bool checkCodecs(const points::AttributeSet::Descriptor& desc,
 
 } // anonymous namespace
 
-PointExecutable::PointExecutable(const std::shared_ptr<const llvm::LLVMContext>& context,
-                const std::shared_ptr<const llvm::ExecutionEngine>& engine,
-                const AttributeRegistry::ConstPtr& attributeRegistry,
-                const CustomData::ConstPtr& customData,
-                const std::unordered_map<std::string, uint64_t>& functions,
-                const ast::Tree& ast)
+PointExecutable::PointExecutable(
+#if LLVM_VERSION_MAJOR <= 15
+        const std::shared_ptr<const llvm::LLVMContext>& context,
+        const std::shared_ptr<const llvm::ExecutionEngine>& engine,
+#else
+        const std::shared_ptr<const llvm::orc::LLJIT>& engine,
+#endif
+        const AttributeRegistry::ConstPtr& attributeRegistry,
+        const CustomData::ConstPtr& customData,
+        const std::unordered_map<std::string, uint64_t>& functions,
+        const ast::Tree& ast)
+#if LLVM_VERSION_MAJOR <= 15
     : mContext(context)
     , mExecutionEngine(engine)
+#else
+    : mExecutionEngine(engine)
+#endif
     , mAttributeRegistry(attributeRegistry)
     , mCustomData(customData)
     , mFunctionAddresses(functions)
     , mSettings(new Settings<false>)
 {
-    assert(mContext);
-    assert(mExecutionEngine);
-    assert(mAttributeRegistry);
+#if LLVM_VERSION_MAJOR <= 15
+    OPENVDB_ASSERT(mContext);
+#endif
+    OPENVDB_ASSERT(mExecutionEngine);
+    OPENVDB_ASSERT(mAttributeRegistry);
 
     // parse the AST for known functions which require pre/post processing
     mSettings->mPostDelete = ast::callsFunction(ast, "deletepoint");
@@ -769,8 +783,12 @@ PointExecutable::PointExecutable(const std::shared_ptr<const llvm::LLVMContext>&
 }
 
 PointExecutable::PointExecutable(const PointExecutable& other)
+#if LLVM_VERSION_MAJOR <= 15
     : mContext(other.mContext)
     , mExecutionEngine(other.mExecutionEngine)
+#else
+    : mExecutionEngine(other.mExecutionEngine)
+#endif
     , mAttributeRegistry(other.mAttributeRegistry)
     , mCustomData(other.mCustomData)
     , mFunctionAddresses(other.mFunctionAddresses)
